﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Windows.Forms;

namespace ZombieEditor {
	public partial class EditorForm : Form {

		#region Constants

		private const string TileDirectory = "Tiles";

		private const string MapFileExtension = "zdx";

		#endregion

		#region Fields

		/// <summary>
		/// Stores the different tile graphics.
		/// </summary>
		
		private Image[] Tiles;

		/// <summary>
		/// Stores the current map being edited.
		/// </summary>
		private Map CurrentMap = new Map();
		

		#endregion

		#region Properties

		private Point editorScrollOffset;
		/// <summary>
		/// Gets or sets the editor scroll offset.
		/// </summary>
		private Point EditorScrollOffset {
			get {
				return editorScrollOffset;
			}
			set {
				this.editorScrollOffset = value;
				this.EditorPanel.Invalidate();
			}
		}

		private Point? editorCursorPosition;
		/// <summary>
		/// Gets or sets the current cursor position inside the editor.
		/// </summary>
		private Point? EditorCursorPosition {
			get {
				return this.editorCursorPosition;
			}
			set {
				this.editorCursorPosition = value;
				this.CursorPositionStatusLabel.Text = this.editorCursorPosition.HasValue 
					? string.Format("({0}, {1})", this.editorCursorPosition.Value.X, this.editorCursorPosition.Value.Y) 
					: "";
			}
		}

		private byte editorCurrentTile = 0;
		/// <summary>
		/// Gets the index of the current editor tile.
		/// </summary>
		private byte EditorCurrentTile {
			get { return this.editorCurrentTile; }
			set { this.editorCurrentTile = value; }
		}

		#endregion

		#region Initialisation

		public EditorForm() {

			this.CurrentMap = new Map();
			this.CurrentMap.WidthChanged += new EventHandler(CurrentMap_SizeChanged);
			this.CurrentMap.HeightChanged += new EventHandler(CurrentMap_SizeChanged);

			InitializeComponent();
			this.Text = Application.ProductName;
			
			this.LoadTileList();
			this.OpenMapDialog.Filter = this.SaveMapDialog.Filter = Application.ProductName + " Files (*." + MapFileExtension + ")|*." + MapFileExtension + "|All Files (*.*)|*.*";
		}

		private void EditorForm_Load(object sender, EventArgs e) {
			this.CurrentMap_SizeChanged(null, null);
			this.MapPropertyGrid.SelectedObject = this.CurrentMap;
		}

		private void LoadTileList() {

			// Clean up old tiles.
			if (this.Tiles != null) {
				for (int i = 0; i < this.Tiles.Length; ++i) {
					this.Tiles[i].Dispose();
				}
			}
			this.TilePicker.Items.Clear();

			// Recreate array.
			this.Tiles = new Image[257];

			// Iterate over all tiles.
			foreach (var Tile in Directory.GetFiles(Path.Combine(Application.StartupPath, TileDirectory), "*.gif")) {
				int FileIndex = 0;
				if (int.TryParse(Path.GetFileNameWithoutExtension(Tile), out FileIndex)) {
					if (this.Tiles[FileIndex] != null) this.Tiles[FileIndex].Dispose();
					var B = new Bitmap(Tile);
					//B.MakeTransparent(Color.Magenta);
					this.Tiles[FileIndex] = B;
				}
			}

			// Dump in the "missing" graphics where appropriate.
			for (int i = 0; i < this.Tiles.Length; ++i) {
				if (this.Tiles[i] == null) {
					this.Tiles[i] = Properties.Resources.MissingTile;
				} else {
					this.TileImageList.Images.Add(this.Tiles[i]);
					this.TilePicker.Items.Add(new ListViewItem(i.ToString(), i) { Tag = i });
				}
			}

		}

		#endregion

		#region Editing

		// Render the editor.
		private void EditorPanel_Paint(object sender, PaintEventArgs e) {
			e.Graphics.PixelOffsetMode = PixelOffsetMode.Half;
			e.Graphics.FillRectangle(Brushes.Gray, e.ClipRectangle);
			for (int y = e.ClipRectangle.Top & ~31; y < e.ClipRectangle.Bottom; y += 32) {
				for (int x = e.ClipRectangle.Left & ~31; x < e.ClipRectangle.Right; x += 32) {
					var TilePosition = new Point(x / 32 + this.editorScrollOffset.X, y / 32 + this.editorScrollOffset.Y);
					if (TilePosition.X>=0 && TilePosition.X < this.CurrentMap.Width && TilePosition.Y >=0 && TilePosition.Y < this.CurrentMap.Height) {
						e.Graphics.DrawImage(this.Tiles[this.CurrentMap[x / 32 + this.editorScrollOffset.X, y / 32 + this.editorScrollOffset.Y]], x, y);
					} else {
						e.Graphics.DrawImage(this.Tiles[256], x, y);
					}
				}
			}
		}

		// Handle editor  mouse move.
		private void EditorPanel_MouseMove(object sender, MouseEventArgs e) {
			var P = new Point(e.X / 32, e.Y / 32);
			P.Offset(this.EditorScrollOffset);
			this.EditorCursorPosition = P;
			if ((e.Button & MouseButtons.Left) != MouseButtons.None) this.PaintTile();
		}

		// Handle editor mouse down.
		private void EditorPanel_MouseDown(object sender, MouseEventArgs e) {
			if ((e.Button & MouseButtons.Left) != MouseButtons.None) this.PaintTile();
		}

		// Handle editor mouse exit.
		private void EditorPanel_MouseLeave(object sender, EventArgs e) {
			this.EditorCursorPosition = null;
		}

		/// <summary>
		/// Paint the current tile under the current mouse position.
		/// </summary>
		private void PaintTile() {
			this.PaintTile(this.EditorCurrentTile);
		}

		/// <summary>
		/// Paint the specified tile under the current mouse position.
		/// </summary>
		private void PaintTile(byte tile) {
			var P = this.EditorCursorPosition;
			if (P.HasValue) {
				if (this.CurrentMap[P.Value.X, P.Value.Y] != tile) {
					this.CurrentMap[P.Value.X, P.Value.Y] = tile;
					this.EditorPanel.Invalidate(new Rectangle((P.Value.X - this.EditorScrollOffset.X) * 32, (P.Value.Y - this.EditorScrollOffset.Y) * 32, 32, 32));
				}
			}
		}


		// Handle selection of a new tile.
		private void TilePicker_SelectedIndexChanged(object sender, EventArgs e) {
			if (this.TilePicker.SelectedItems.Count == 1) {
				this.EditorCurrentTile = (byte)(int)this.TilePicker.SelectedItems[0].Tag;
			}
		}

		#endregion
		
		#region Scrolling

		void CurrentMap_SizeChanged(object sender, EventArgs e) {
			this.EditorScrollH.Value = Math.Min(this.CurrentMap.Width + this.EditorScrollH.LargeChange, this.EditorScrollH.Value);
			this.EditorScrollH.Maximum = this.CurrentMap.Width + this.EditorScrollH.LargeChange;
			this.EditorScrollV.Value = Math.Min(this.CurrentMap.Height + this.EditorScrollV.LargeChange, this.EditorScrollV.Value);
			this.EditorScrollV.Maximum = this.CurrentMap.Height + this.EditorScrollV.LargeChange;
			this.EditorPanel.Invalidate();
		}

		private void EditorScroll_ValueChanged(object sender, EventArgs e) {
			this.EditorScrollOffset = new Point(this.EditorScrollH.Value, this.EditorScrollV.Value);
		}

		#endregion

		#region Saving and Loading

		private string currentFilename = null;
		private string CurrentFilename {
			get { return this.currentFilename; }
			set {
				this.currentFilename = value;
				this.CurrentFilenameStatusLabel.Text = this.currentFilename ?? "";
			}
		}

		private bool SaveAs() {
			if (this.SaveMapDialog.ShowDialog(this) == DialogResult.OK) {
				try {
					this.CurrentMap.Save(this.SaveMapDialog.FileName);
					this.CurrentFilename = this.SaveMapDialog.FileName;
					return true;
				} catch (Exception ex) {
					MessageBox.Show("Could not save file: " + ex.Message);
				}
			}
			return false;
		}

		private bool Save() {
			if (this.CurrentFilename == null) {
				return this.SaveAs();
			} else {
				try {
					this.CurrentMap.Save(this.CurrentFilename);
					return true;
				} catch (Exception ex) {
					MessageBox.Show("Could not save file: " + ex.Message);
				}
			}
			return false;
		}


		private bool Open() {
			if (this.OpenMapDialog.ShowDialog(this) == DialogResult.OK) {
				try {
					this.CurrentMap.Load(this.OpenMapDialog.FileName);
					this.EditorPanel.Invalidate();
					this.MapPropertyGrid.Refresh();
					this.CurrentFilename = this.OpenMapDialog.FileName;
					return true;
				} catch (Exception ex) {
					MessageBox.Show("Could not open file: " + ex.Message);
				}
			}
			return false;
		}

		private void OpenMenu_Click(object sender, EventArgs e) {
			this.Open();
		}

		private void SaveMenu_Click(object sender, EventArgs e) {
			this.Save();
		}

		private void SaveAsMenu_Click(object sender, EventArgs e) {
			this.SaveAs();
		}

		#endregion

		


		

	}
}
