﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.IO;

namespace ZombieEditor {
	public class Map {

		/// <summary>
		/// Save the <see cref="Map"/> to a file.
		/// </summary>
		/// <param name="filename">The name of the file to save to.</param>
		public void Save(string filename) {
			using (var s = File.OpenWrite(filename)) {
				this.Save(s);
			}
		}

		/// <summary>
		/// Save the <see cref="Map"/> to a <see cref="Stream"/>.
		/// </summary>
		/// <param name="stream">The <see cref="Stream"/> to save to.</param>
		public void Save(Stream stream) {
			var bw = new BinaryWriter(stream);
			bw.Write(this.Height);
			bw.Write(this.Width); 
			for (int y = 0; y < this.height; ++y) {
				bw.Write(this.data[y], 0, this.width);
			}
			bw.Flush();
		}

		/// <summary>
		/// Load the <see cref="Map"/> from a file.
		/// </summary>
		/// <param name="filename">The name of the file to load from.</param>
		public void Load(string filename) {
			using (var s = File.OpenRead(filename)) {
				this.Load(s);
			}
		}

		/// <summary>
		/// Loads the <see cref="Map"/> from a <see cref="Stream"/>.
		/// </summary>
		/// <param name="stream">The <see cref="Stream"/> to load from.</param>
		public void Load(Stream stream) {
			var br = new BinaryReader(stream);
			this.height = br.ReadInt32();
			this.width = br.ReadInt32();
			this.data = new byte[this.height][];
			for (int i = 0; i < this.height; ++i) {
				this.data[i] = br.ReadBytes(this.width);
			}
			this.Width = this.Width;
			this.Height = this.Height;
		}

		private int width = 16;
		[Category("Dimensions")]
		public int Width {
			get { return this.width; }
			set {
				if (value < 1) throw new Exception("Width must be at least 1.");
				for (int i = 0; i < this.height; ++i) {
					if (value > this.data[i].Length) {
						Array.Resize(ref this.data[i], value);
					}
				}
				this.width = value;
				this.OnWidthChanged(new EventArgs());
			}
		}

		public event EventHandler WidthChanged;
		protected void OnWidthChanged(EventArgs e) {
			if (this.WidthChanged != null) this.WidthChanged(this, e);
		}

		private int height = 16;
		[Category("Dimensions")]
		public int Height {
			get { return this.height; }
			set {
				if (value < 1) throw new Exception("Height must be at least 1.");
				if (value > this.data.Length) {
					int OldLength = this.data.Length;
					Array.Resize(ref this.data, value);
					for (int i = OldLength; i < value; ++i) {
						this.data[i] = new byte[this.width];
					}
				}
				this.height = value;
				this.OnHeightChanged(new EventArgs());
			}
		}

		public event EventHandler HeightChanged;
		protected void OnHeightChanged(EventArgs e) {
			if (this.WidthChanged != null) this.HeightChanged(this, e);
		}

		public Map() {
			this.data = new byte[this.Height][];
			for (int i = 0; i < this.data.Length; ++i) {
				this.data[i] = new byte[this.Width];
			}
		}

		private byte[][] data;

		public byte this[int x, int y] {
			get {
				return x >= 0 && x < this.width && y >= 0 && y < this.height ? this.data[y][x] : (byte)0;
			}
			set {
				if (x >= 0 && x < this.width && y >= 0 && y < this.height) this.data[y][x] = value;
			}
		}

	}
}
